#!/usr/bin/perl
use strict;
use Getopt::Long;

use constant {
    FAIL    => 0,
    SUCCESS => 1,

    FILE_PATCH_INFO => 'patch.info',

    CFG_LOG_PATH   => '/webmail/log/fix_heartbleed.log',

    CMD_RESTARTSHM => '/webmail/tools/restartshm',
};

my $M2K_DAEMON_TOOL = "/webmail/tools/m2kctrl";
my $sg_szCmdMd5 = (-x '/bin/md5sum')?'/bin/md5sum':'/usr/bin/md5sum';

sub s_CheckDaemon {
    my $daemon_name = shift;
    my ($ret, $ret_status);

    print "Cannot execute mail2000 daemon control tool.\n" if(! -x $M2K_DAEMON_TOOL);

    $ret = `$M2K_DAEMON_TOOL -c status -s $daemon_name`;

    print "Unknown status from mail2000 daemon control tool.\n" if($ret !~ /is (stop|running|partial running)/);

    $ret_status = $1;

    return if($ret_status eq "stop");

    print "Daemon $daemon_name is running.\n";
}

sub s_CtrlDaemon 
{
    my ($szAction, $daemon_name) = @_;
    my ($ret, $ret_status);

    print "Cannot execute mail2000 daemon control tool.\n" if(! -x $M2K_DAEMON_TOOL);

    $ret = `$M2K_DAEMON_TOOL -c $szAction -s $daemon_name`;

    print "Control mail2000 daemon fail: $daemon_name.\n" if($ret !~ /$szAction $daemon_name...DONE/);

    $ret_status = $1;

    return if($ret_status eq "stop");

    print "Daemon $daemon_name is $szAction.\n";
}

sub s_CheckMd5
{
    my ($szFileList) = @_;
    unless( open( FH_PATCH_INFO_R, "< ", $szFileList )) {
        s_LogStr("ERROR", "Open file fail: $szFileList" );
        print "ERROR - Open file fail: $szFileList\n";
    }
    foreach my $szLine ( <FH_PATCH_INFO_R> ) {
        chomp($szLine);
        my ($szMd5, $szFile) = $szLine =~ /(\S+)\s+(\S+)/;
        my $szCmd = sprintf("%s %s", $sg_szCmdMd5, $szFile);
        my $szMd5Info = `$szCmd`;
        chomp($szMd5Info);
        my ($szMd5File) = $szMd5Info =~ /(\S+)\s+\S+/;
        if( $szMd5File ne $szMd5) {
            s_LogStr("ERROR", "Md5 check fail: $szFile");
            print "ERROR - Md5 check fail: $szFile\n";
            exit(1);
        }
    }
    close(FH_PATCH_INFO_R);
}

sub Main 
{
    my $nRet = 0;
    my ($szTemplate, $szPortalSimp) = ('', '');

    # 1. Check user.
    s_CheckUser();

    # 2. Get patch info and check md5 of patch files.
    s_LogStr("INFO", "Check md5 of patch files");
    print "Check md5 of patch files.\n";
    s_CheckMd5(FILE_PATCH_INFO);
    s_LogStr("INFO", "Check md5 of patch file success.");
    print "Check md5 of patch file success.\n";

    # 3. Backup original files.
    system('./do_backup.pl m2kpatch_backup_mp601404101143 patch_backup.lst');

    # 4. Stop httpd daemon.
    s_LogStr("INFO", "Stop httpd daemon");
    print "Stop httpd daemon.\n";
    s_CtrlDaemon("stop", "httpd");

    # 5. Copy apache files to mail2000.
    s_LogStr("INFO", "Copy new apache files to mail2000 system");
    print "Copy new apache files to mail2000 system.\n";
    system("cp -prf httpd /webmail/");

    # 6. Start httpd daemon.
    s_LogStr("INFO", "Start httpd daemon");
    print "Start httpd daemon.\n";
    s_CtrlDaemon("start", "httpd");

    s_LogStr("INFO", "Fix heartbleed completely");
    print "Fix heartbleed completely.\n";
}

# Print out the log
sub s_LogStr
{
    my ($state, $content) = @_;
    my @time = localtime(time);

    open(LOGFILE, ">>", CFG_LOG_PATH );

    my @log = split(/\n/, $content);
    for (my $i=0; $i <= $#log; $i++) {
        printf LOGFILE "[%d/%d/%d %02d:%02d:%02d] [%s] - %s\n", $time[5] + 1900, $time[4] + 1, $time[3],
            $time[2], $time[1], $time[0], $state, $log[$i];
    }

    close(LOGFILE);
}

sub s_CheckUser 
{
    my ($szUser1, $szUser2);
    $szUser1 = getpwuid ($<);
    $szUser2 = getpwuid ($>);
    if (($szUser1 ne 'webmail') or ($szUser2 ne 'webmail')) {
        print ("ERROR: Please run as webmail. Exit\n");
        exit;
    }
}

&Main();
